# coding: utf-8
"""Storj metadata module."""

__author__ = ','.join([
    'Daniel Hawkins',
    'Fabian Barkhau',
    'James Prestwich',
    'Marc Schubert',
    'Marco Rosa',
    'Mike Bailey',
    'Pedro Salgado',
    'Shawn Wilkinson',
    'Wiktor Jezioro'
])
__author_email__ = ';'.join([
    'hwkns@alum.mit.edu',
    'f483@storj.io',
    'james@storj.io',
    'marc.schubert@gmail.com',
    'marcor165@hotmail.it',
    'mibgranny@aol.com',
    'steenzout@ymail.com',
    'shawn+pythonsdk@storj.io',
    'lakewik@lakewik.pl'
])
__version__ = '0.3.0'
